// src/index.ts
import { createUnplugin } from "unplugin";
import { createFilter } from "@rollup/pluginutils";

// src/core/options.ts
function resolveOption(options) {
  return {
    include: options.include || [/\.(vue|[cm]?[jt]sx?)$/],
    exclude: options.exclude || [/node_modules/],
    enforce: "enforce" in options ? options.enforce : "pre"
  };
}

// src/core/transform/index.ts
import {
  MagicString,
  babelParse,
  getLang,
  getTransformResult,
  parseSFC
} from "@vue-macros/common";

// src/core/transform/v-if.ts
import { walkAST } from "@vue-macros/common";
function vIfTransform(ast, s, offset = 0) {
  if (!s.slice(ast.start + offset, ast.end + offset).includes("v-if"))
    return;
  const nodeMap = /* @__PURE__ */ new Map();
  walkAST(ast, {
    enter(node, parent) {
      if (node.type !== "JSXElement")
        return;
      const attribute = node.openingElement.attributes.find(
        (i) => i.type === "JSXAttribute" && ["v-if", "v-else-if", "v-else"].includes(`${i.name.name}`)
      );
      if (attribute) {
        if (!nodeMap.has(parent))
          nodeMap.set(parent, []);
        nodeMap.get(parent)?.push({
          node,
          attribute
        });
      }
    }
  });
  const nodes = [...nodeMap.values()].flat();
  nodes.forEach(({ node, attribute }, index) => {
    if (["v-if", "v-else-if"].includes(`${attribute.name.name}`)) {
      if (attribute.value)
        s.appendLeft(
          node.start + offset,
          `${attribute.name.name === "v-if" ? "{ " : ""}${s.slice(
            attribute.value.start + offset + 1,
            attribute.value.end + offset - 1
          )} ? `
        );
      s.appendRight(
        node.end + offset,
        `${nodes[index + 1]?.attribute.name.name}`.startsWith("v-else") ? " :" : " : ''}"
      );
      s.remove(attribute.start + offset - 1, attribute.end + offset);
    } else {
      s.appendRight(node.end + offset, " }");
    }
  });
  return s;
}

// src/core/transform/v-for.ts
import { walkAST as walkAST2 } from "@vue-macros/common";
function vForTransform(ast, s, offset = 0) {
  if (!s.slice(ast.start + offset, ast.end + offset).includes("v-for"))
    return;
  const nodes = [];
  walkAST2(ast, {
    enter(node) {
      if (node.type !== "JSXElement")
        return;
      const attribute = node.openingElement.attributes.find(
        (i) => i.type === "JSXAttribute" && ["v-for"].includes(`${i.name.name}`)
      );
      if (attribute) {
        nodes.push({
          node,
          attribute
        });
      }
    }
  });
  nodes.forEach(({ node, attribute }) => {
    if (`${attribute.name.name}` === "v-for") {
      if (!attribute.value)
        return;
      const [i, , list] = s.slice(
        attribute.value.start + offset + 1,
        attribute.value.end + offset - 1
      ).split(/\s/);
      s.appendLeft(node.start + offset, ` { ${list}.map(${i}=>`);
      s.appendRight(node.end + offset, ") }");
      s.remove(attribute.start + offset - 1, attribute.end + offset);
    }
  });
  return s;
}

// src/core/transform/index.ts
function transform(code, id) {
  const lang = getLang(id);
  let asts = [];
  if (lang === "vue") {
    const { scriptSetup, getSetupAst, script, getScriptAst } = parseSFC(
      code,
      id
    );
    if (script) {
      asts.push({ ast: getScriptAst(), offset: script.loc.start.offset });
    }
    if (scriptSetup) {
      asts.push({ ast: getSetupAst(), offset: scriptSetup.loc.start.offset });
    }
  } else if (["jsx", "tsx"].includes(lang)) {
    asts = [{ ast: babelParse(code, lang), offset: 0 }];
  } else {
    return null;
  }
  const s = new MagicString(code);
  for (const { ast, offset } of asts) {
    vIfTransform(ast, s, offset);
    vForTransform(ast, s, offset);
  }
  return getTransformResult(s, id);
}

// src/index.ts
var src_default = createUnplugin((rawOptions = {}) => {
  const options = resolveOption(rawOptions);
  const filter = createFilter(options.include, options.exclude);
  const name = "unplugin-jsx-vue-directive";
  return {
    name,
    enforce: options.enforce,
    transformInclude(id) {
      return filter(id);
    },
    transform
  };
});

export {
  src_default
};
